# Copyright 2021 Camptocamp SA
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from datetime import datetime

from odoo import api, models


class SaleOrder(models.Model):
    _inherit = "sale.order"

    @api.model
    def cron_generate_weekly_invoices(self):
        """Cron called daily to check if weekly invoicing needs to be done."""
        company_ids = self._company_weekly_invoicing_today()
        if company_ids:
            self.generate_weekly_invoices(company_ids)

    @api.model
    def generate_weekly_invoices(self, companies=None):
        """Generate weekly invoices for customers who require that mode.

        Invoices will be generated by other jobs split for different customer
        and different payment term.
        """
        if not companies:
            companies = self.company_id
        saleorder_groups = self.read_group(
            [
                ("invoicing_mode", "=", "weekly"),
                ("invoice_status", "=", "to invoice"),
                ("company_id", "in", companies.ids),
            ],
            ["partner_invoice_id"],
            groupby=self._get_groupby_fields_for_weekly_invoicing(),
            lazy=False,
        )
        for saleorder_group in saleorder_groups:
            saleorder_ids = self.search(saleorder_group["__domain"]).ids
            self.with_delay()._generate_invoices_by_partner(saleorder_ids)
        companies.write({"invoicing_mode_weekly_last_execution": datetime.now()})
        return saleorder_groups

    @api.model
    def _get_groupby_fields_for_weekly_invoicing(self):
        """Returns the sale order fields used to group them into jobs."""
        return ["partner_invoice_id", "payment_term_id"]

    def _generate_invoices_by_partner(self, saleorder_ids, invoicing_mode="weekly"):
        """Generate invoices for a group of sale order belonging to a customer."""
        sales = (
            self.browse(saleorder_ids)
            .exists()
            .filtered(lambda r: r.invoice_status == "to invoice")
        )
        if not sales:
            return "No sale order found to invoice ?"
        invoices = sales._create_invoices(
            grouped=sales[0].partner_invoice_id.one_invoice_per_order,
            final=True,
        )
        for invoice in invoices:
            invoice.with_delay()._validate_invoice()
        return invoices

    @api.model
    def _company_weekly_invoicing_today(self):
        """Get company ids for which today is weekly invoicing day."""
        today = datetime.now()
        domain = [
            "|",
            ("invoicing_mode_weekly_last_execution", "<", today),
            ("invoicing_mode_weekly_last_execution", "=", False),
            ("invoicing_mode_weekly_day_todo", "=", today.weekday()),
        ]
        return self.env["res.company"].search(domain)
